<?php
/**
 * Importa os valores da tabela de integração(IntgNotafiscal) para a real (Notafiscal).
 * Exemplo de codigo de uso:
 * $importador = new Fiscal\Logic\ImportacaoNFE\ImportadorNFE($nrorg, $cdForneced, $cdSerie, $nrNotaFisc);
 * $importador->importaDocumento($cdTipoOperacao);
 * ou
 * Fiscal\Logic\ImportacaoNFEImportadorNFE::importa($nrorg, $cdForneced, $cdSerie, $nrNotaFisc, $cdTipoOperacao);
 *
 */

namespace Fiscal\Logic\ImportacaoNFE;

use \Fiscal\Entities\Imposnftot;
use \Fiscal\Entities\IntgItemencargo;
use \Fiscal\Entities\IntgItemproduto;
use \Fiscal\Entities\IntgNotafiscal;
use \Fiscal\Entities\IntgTrabitproimpos;
use \Fiscal\Entities\Itemproimpos;
use \Fiscal\Entities\Notafiscal;
use \Geral\Entities\Encargo;
use \Geral\Entities\Filial;
use \Geral\Entities\Fornecedor;
use \Geral\Entities\Imposto;
use \Geral\Entities\Itemencargo;
use \Geral\Entities\Itemproduto;
use \Geral\Entities\Produto;
use \Indeterminado\Entities\Prodfornxml;
use \Setforms2\Runtime\Helper\Procedure;

/**
 * Executa a importacao dos registros das tabelas INTG_*, referentes a nota fiscal para as tabelas de Nota Fiscal.
 */
class ImportadorNFE
{
    private $valorImpostoTotalizado = array();
    private $valorBaseTotalizado = array();
    /** @var null|\Fiscal\Entities\IntgNotafiscal */
    private $nfe = null;
    /** @var null|\Fiscal\Entities\Notafiscal*/
    private $notaFiscal = null;
    private $itemNaoAssociados = array();

    /**
     * @var $filialProcessamento Filial
     */
    private $filialProcessamento;
    /**
     * @var $filialRecebimento Filial
     */
    private $filialRecebimento;

    /**
     * @param $nrorg
     * @param $cdForneced
     * @param $cdSerie
     * @param $nrNotaFisc
     */
    public function __construct($nrorg, $cdForneced, $cdSerie, $nrNotaFisc)
    {
        /** @var $repositorioIntgNotafiscal \Fiscal\Repositories\IntgNotafiscal */
        $repositorioIntgNotafiscal = \Fiscal\Entities\IntgNotafiscal::getRepository();

        $this->nfe = $repositorioIntgNotafiscal->retornaChaveAcessoDocumento (
            $nrorg, $cdForneced, $cdSerie, $nrNotaFisc
        );

        $this->filialProcessamento = \Geral\Entities\Filial::getObject (
            array(
                "cdfilial" => \Geral\Util\Ambiente::getCdFilial(),
                "nrorg"    => \Geral\Util\Ambiente::getNrorg(),
            )
        );
        $this->filialRecebimento   = \Geral\Entities\Filial::getObject (
            array(
                "cdfilial" => \Geral\Util\Ambiente::getCdFilial(),
                "nrorg"    => \Geral\Util\Ambiente::getNrorg(),
            )
        );
    }

    protected function validaNFE()
    {
        if($this->nfe === null) throw new \Exception("Nenhuma NF a ser importada.");
        if($this->nfe->foiImportada())
            throw new \Fiscal\Logic\ImportacaoNFE\Exceptions\NFEImportada();
    }

    protected function alteraStatusNFE()
    {
        $this->nfe->marcarNotaFiscalComoImportada($this->notaFiscal);
        $this->nfe->persist();
    }

    /**
     * @param $cdTipoOperacao
     * @return \Fiscal\Entities\Notafiscal|null
     * @throws \Exception
     */
    public function importaDocumento($cdTipoOperacao)
    {
        $this->validaNFE();
        $connection = \Setforms2\Runtime\Registry::getEntityManagerConnection();
        $connection->beginTransaction();
        try {
            $this->importaNotaFiscal($cdTipoOperacao);
            $this->importaProdutos();
            $this->importaEncargos();
            $this->totalizaImpostos();
            $this->alteraStatusNFE();
            \Setforms2\Runtime\Registry::getEntityManager()->flush();
            $connection->commit();
            return $this->notaFiscal;
        } catch (\Exception $e) {
            $connection->rollback();
            throw $e;
        }
    }

    /**
     * @param $cdTipoOperacao
     */
    protected function importaNotaFiscal($cdTipoOperacao)
    {
        $this->notaFiscal = new Notafiscal();
        $this->notaFiscal->setFilial($this->filialProcessamento);
        $this->notaFiscal->setNrlanctonf($this->getNumeroLancamento());
        $this->notaFiscal->setFilirece($this->filialRecebimento);
        // $this->notaFiscal->setEmpresa($this->nfe->getEmpresa());
        $this->notaFiscal->setEmpresa($this->getEmpresa());
        $this->notaFiscal->setInscesta($this->getInscricaoEstadual());
        $this->notaFiscal->setIdtpijurnf($this->nfe->getIdtpijurnf());
        $this->notaFiscal->setNrinsjurnf($this->nfe->getNrinsjurnf());
        $this->notaFiscal->setForneced($this->nfe->getFornecedor());
        $this->notaFiscal->setModdocfisc($this->nfe->getModeloDocumentoFiscal());
        $this->notaFiscal->setIdbloqueado(Notafiscal::NOTA_FISCAL_NAO_BLOQUEADA);
        $this->notaFiscal->setIddocfiscal(Notafiscal::DOCUMENTO_FISCAL);
        $this->notaFiscal->setIddocfinanc(Notafiscal::DOCUMENTO_FINANCEIRO);
        $this->notaFiscal->setNrnotafisc($this->nfe->getNrnotafisc());
        $this->notaFiscal->setDtemissao($this->nfe->getDtemissao());
        $this->notaFiscal->setDtentrsaid($this->nfe->getDtentrsaid());
        $this->notaFiscal->setVrprodutos($this->nfe->getVrprodutos());
        $this->notaFiscal->setVrnotafisc($this->nfe->getVrnotafisc());
        $this->notaFiscal->setEntrsaidop($this->getTipoOperacao($cdTipoOperacao));
        $this->notaFiscal->setSerieentr($this->nfe->getSerieEntrada());
        $this->notaFiscal->setIdstatus(Notafiscal::APROVADO);
        $this->notaFiscal->setIdsistema(Notafiscal::PREFIXO_SISTEMA_DFE);
        $this->notaFiscal->setIdorigdestnf($this->nfe->getIdorigdestnf());
        $this->notaFiscal->setDtlancamennf($this->nfe->getDtlancamennf());
        $this->notaFiscal->setDtcompfatur(new \DateTime());
        $this->notaFiscal->setNracessonfe($this->nfe->getNracessonfe());
        $this->notaFiscal->setIdstatusnfe($this->nfe->getIdstatusnfe());
        $this->notaFiscal->setIdimpressodanfe(Notafiscal::DOCUMENTO_NAO_IMPRESSO);
        $this->notaFiscal->setIdtipoemisnfe($this->nfe->getIdtipoemisnfe());
        $this->notaFiscal->setIdexpnfep(Notafiscal::NFE_PAULISTANA_NAO);
        $this->notaFiscal->setIdexpnfesanand(Notafiscal::IDEXPNFESANAND_NAO);
        $this->notaFiscal->setIdexpnfemanaus(Notafiscal::NFE_MANAUS_NAO);
        $this->notaFiscal->setIdemailnfe(Notafiscal::ENVIO_EMAIL_TRANSPORTADORA_PENDENTE);
        $this->notaFiscal->setIdstatusctbnf(Notafiscal::STATUS_CONTABILIZACAO_PENDENTE);
        $this->notaFiscal->setDtinclusao(new \Datetime());
        $this->notaFiscal->setIdverifnfconf(Notafiscal::NOTA_NAO_CONFERIDA);
        $this->notaFiscal->setIdnfdispfin(Notafiscal::DISPONIVEL_PARA_FINANCEIRO);
        $this->notaFiscal->setIdtpambnfe(Notafiscal::IDTPAMBNFE_0);
        $this->notaFiscal->setIdemailnfetransp(\Fiscal\Entities\Notafiscal::ENVIO_EMAIL_TRANSPORTADORA_PENDENTE);
        $this->notaFiscal->setIdbloqnffin(Notafiscal::NF_LIBERADA_NO_FINANCEIRO);
        $this->notaFiscal->setIdnfgeradaviaxml(Notafiscal::NF_GERADA_VIA_XML);
        $this->notaFiscal->setIdnfcompraleg(Notafiscal::NF_COMPRA_LEGAL_NAO);
        $this->notaFiscal->setIddebicrednf(\Fiscal\Entities\Notafiscal::ID_DEBITO_NENHUM_NOTA_FISCAL);
        $this->notaFiscal->setIdstatusnfe(\Fiscal\Entities\Notafiscal::ID_STATUS_NFE_PENDENTE);
        $this->notaFiscal->persist();
        \Setforms2\Runtime\Registry::getEntityManager()->flush();
    }

    /**
     * @param $cdTipoOperacao
     * @return object
     */
    protected function getTipoOperacao($cdTipoOperacao)
    {
        $tipooperacId = array(
            "cdtipooper"   => $cdTipoOperacao,
            "nrorg"        => \Geral\Util\Ambiente::getNrorg(),
            "identrsaidop" => \Geral\Entities\Tipooperac::OPERACAO_ENTRADA,
        );

        return \Geral\Entities\Tipooperac::getObject($tipooperacId);
    }

    protected function importaProdutos()
    {
        $itensNFE = IntgItemproduto::findByNracessonfeAndNrorg (
            $this->nfe->getNracessonfe(), $this->nfe->getNrorg()
        );
        foreach ($itensNFE as $itemNFE) {
            /**  @var $itemNFE \Fiscal\Entities\IntgItemproduto */
            $this->importaProduto($itemNFE);
        }

        $this->validaItensNaoAssociados();
    }

    /**
     * @param $itemNFE
     */
    protected function acumulaItemNaoAssociado($itemNFE)
    {
        $this->itemNaoAssociados[] = $itemNFE;
    }

    protected function validaItensNaoAssociados()
    {
        if (count($this->itemNaoAssociados))
            throw new \Fiscal\Logic\ImportacaoNFE\Exceptions\ItemNaoAssociado($this->itemNaoAssociados);
    }

    /**
     * @param \Fiscal\Entities\IntgItemproduto $itemNFE
     */
    protected function importaProduto(IntgItemproduto $itemNFE)
    {
        $itemAssociado = null;
        if ($this->nfe->getFornecedor() !== null) {
            $itemAssociado = Prodfornxml::findOneByCdprodutoxmlAndCdfornecedAndNrorg (
                $itemNFE->getCdproduto(),
                $this->nfe->getFornecedor()->getCdforneced(),
                \Geral\Util\Ambiente::getNrorg()
            );
        }

        if($itemAssociado) {
             $this->importaProdutoComplemento($itemNFE, $itemAssociado);
        } else {
             $this->acumulaItemNaoAssociado($itemNFE);
        }
    }

    /**
     * @param \Fiscal\Entities\IntgItemproduto $itemNFE
     * @param \Indeterminado\Entities\Prodfornxml $itemAssociado
     */
    protected function importaProdutoComplemento(IntgItemproduto $itemNFE, Prodfornxml $itemAssociado)
    {
        $itemProduto = new Itemproduto();
        $itemProduto->setLanctonf($this->notaFiscal);
        $itemProduto->setNrseqitprod($itemNFE->getNrseqitprod());
        $itemProduto->setProduto($itemAssociado->getProduto());
        $itemProduto->setCdprodemis($itemNFE->getCdproduto());
        $itemProduto->setDsprodemis($itemNFE->getNmproduto());
        $itemProduto->setIdatuvrestoq(Itemproduto::NAO_ATUALIZA_VALOR_ESTOQUE);
        $itemProduto->setQtitemprod($itemNFE->getQtitemprod());
        $itemProduto->setVrunititem($itemNFE->getVrunititem());
        $itemProduto->setVrdescitem($itemNFE->getVrdescitem());
        $itemProduto->setVrtotaitem($itemNFE->getVrtotaitem());
        $itemProduto->setVrcontitem($itemNFE->getVrcontitem() ? $itemNFE->getVrcontitem() : 0);
        $itemProduto->setVrbrutitem($itemNFE->getVrbrutitem() ? $itemNFE->getVrbrutitem() : 0);
        $itemProduto->setIdconforitem(Itemproduto::ITEM_COM_CONFORMIDADE);
        $itemProduto->setDtinclusao(new \DateTime());
//        $itemProduto->setLocalestoq($itemNFE->getLocalEstoque());
        $itemProduto->setCdlocalestoq(\Geral\Util\Ambiente::getInstancia()->getFilial()->getEstoquePadrao());
        $itemProduto->setCdfisopeitem($itemNFE->getCdfisopeitem());
        $itemProduto->setCdsitutribpr($itemNFE->getCdsitutribpr());
        $itemProduto->persist();

        \Setforms2\Runtime\Registry::getEntityManager()->flush();
        $this->importaItemProdImposto($itemProduto, $itemNFE);
    }

    /**
     * @param \Geral\Entities\Itemproduto $itemProduto
     * @param \Fiscal\Entities\IntgItemproduto $itemNFE
     */
    protected function importaItemProdImposto($itemProduto, $itemNFE)
    {
        $itemProdutoImpostosNFE = IntgTrabitproimpos::findByNracessonfeAndNrseqimpoitAndNrorg (
            $itemNFE->getNracessonfe(), $itemNFE->getNrseqitprod(), \Geral\Util\Ambiente::getNrorg()
        );

        /** @var $itemProdutoImpostoNFE IntgTrabitproimpos */
        foreach ($itemProdutoImpostosNFE as $itemProdutoImpostoNFE) {
            $itemProdutoImposto = new Itemproimpos();

            $itemProdutoImposto->setCdfilial($itemProduto->getCdfilial());
            $itemProdutoImposto->setNrlanctonf($itemProduto->getNrlanctonf());
            $itemProdutoImposto->setNrseqitprod($itemProdutoImpostoNFE->getNrseqimpoit());
            $itemProdutoImposto->setCdimposto($itemProdutoImpostoNFE->getCdimpostoit());
            $itemProdutoImposto->setNritproimpos($itemProdutoImpostoNFE->getNritproimpos());
            $itemProdutoImposto->setNrorg(\Geral\Util\Ambiente::getNrorg());

            $itemProdutoImposto->setVrcontitimp($itemProdutoImpostoNFE->getVrcontitimp());
            $itemProdutoImposto->setVrperredbcit($itemProdutoImpostoNFE->getVrperredbcit());
            $itemProdutoImposto->setVrbasecalcit($itemProdutoImpostoNFE->getVrbasecalcit());
            $itemProdutoImposto->setVraliqitem($itemProdutoImpostoNFE->getVraliqitem());
            $itemProdutoImposto->setVrpardedit('1');
            $itemProdutoImposto->setVrimpostoit($itemProdutoImpostoNFE->getVrimpostoit());
            $itemProdutoImposto->setDtinclusao($itemProdutoImpostoNFE->getDtinclusao());
            $itemProdutoImposto->setCdfiscoperit($itemProdutoImpostoNFE->getCdfiscoperit());
            $itemProdutoImposto->setDtinclusao(new \DateTime());
            $itemProdutoImposto->setVrcontitimp(0);
            $itemProdutoImposto->setVrperredbcit(0);
            $itemProdutoImposto->setIddifaliqcuspri(\Fiscal\Entities\Itemproimpos::NAO_UTILIZA_DIFERENCIAL_DE_ALIQUOTA);
            $itemProdutoImposto->setIdimpdesonerait(\Fiscal\Entities\Itemproimpos::IMPOSTO_NAO_DESONERADO_PARA_ITEM);
            $itemProdutoImposto->setIdmotdesonerait(Itemproimpos::DESONERACAO_MOTIVO_OUTROS);
            $itemProdutoImposto->persist();

            $this->somaValorImposto($itemProdutoImposto);
        }
    }

    /**
     * @param \Fiscal\Entities\Itemproimpos $itemProdutoImposto
     */
    protected function somaValorImposto(Itemproimpos $itemProdutoImposto)
    {
        $incrementaPosicaoArray = function (array &$variavel, $chave, $valor) {
            if(!isset($variavel[$chave])) $variavel[$chave] = 0;
            $variavel[$chave] += $valor;
        };

        $codigoImposto = $itemProdutoImposto->getImposto()->getCdimposto();
        $incrementaPosicaoArray($this->valorBaseTotalizado, $codigoImposto, $itemProdutoImposto->getVrbasecalcit());
        $incrementaPosicaoArray($this->valorImpostoTotalizado, $codigoImposto, $itemProdutoImposto->getVrimpostoit());
    }

    protected function importaEncargos()
    {
        $itemEncargosNFE = IntgItemencargo::findByNracessonfeAndNrorg(
            $this->nfe->getNracessonfe(),
            $this->nfe->getNrorg()
        );
        foreach ($itemEncargosNFE as $itemEncargoNFE) {
            $this->importaEncargo($itemEncargoNFE);
        }
    }

    /**
     * @param \Fiscal\Entities\IntgItemencargo $itemEncargoNFE
     */
    protected function importaEncargo(IntgItemencargo $itemEncargoNFE)
    {
        $itemEncargo = new Itemencargo();
        $itemEncargo->setLanctonf($this->notaFiscal);
        $itemEncargo->setEncargo($itemEncargoNFE->getEncargo());
        $itemEncargo->setVritemenca($itemEncargoNFE->getVritemenca());
        $itemEncargo->setDtinclusao(new \DateTime());
        $itemEncargo->persist();
    }

    protected function totalizaImpostos()
    {
        foreach ($this->valorBaseTotalizado as $codigoImposto => $valorBaseTotalizado) {
            $valorImpostoTotalizado = $this->valorImpostoTotalizado[$codigoImposto];

            $impostoTotal = new Imposnftot();
            $impostoTotal->setLanctonf($this->notaFiscal);
            $impostoTotal->setImposto(Imposto::getObject(
                array(
                    "cdimposto" => $codigoImposto,
                    "nrorg"     => \Geral\Util\Ambiente::getNrorg()
                )
            ));
            $impostoTotal->setVrtotimponf($valorImpostoTotalizado);
            $impostoTotal->setVrtotbcalcnf($valorBaseTotalizado);
            $impostoTotal->setCdoperinclusao(\Geral\Util\Ambiente::getCdOperador());
            $impostoTotal->setDtinclusao(new \DateTime());
            $impostoTotal->persist();
        }
    }

    /**
     * Retorna o valor da chave nrlanctonf
     *
     * @return int|string
     */
    protected function getNumeroLancamento()
    {
        $cdcontadorNF = Notafiscal::CDCONTADOR_NOTAFISCAL . \Geral\Util\Ambiente::getCdFilial();
        return \Geral\Util\NovoCodigo::geraCodigo(\Geral\Util\Ambiente::getNrorg(), $cdcontadorNF);
    }

    /**
     * Retorna a instancia da empresa corrente (empresa da filial que esta operando o sistema).
     *
     * @return object
     */
    protected function getEmpresa()
    {
        return \Geral\Entities\Empresa::getObject(
           array (
               "cdempresa" => \Geral\Util\Ambiente::getCdEmpresa(),
               "nrorg"     => \Geral\Util\Ambiente::getNrorg(),
           )
        );
    }

    /**
     * Retorna a instancia da inscricao estadual da empresa corrente.
     *
     * @return object
     */
    protected function getInscricaoEstadual()
    {
        return \Geral\Util\Ambiente::getInstancia()->getFilial()->getInscesta();
    }

    /**
     * Retorna a instancia do objeto de imposto.
     *
     * @param \Fiscal\Entities\IntgTrabitproimpos $itemProdutoImpostoNFE
     * @return object
     */
    protected function getimposto(IntgTrabitproimpos $itemProdutoImpostoNFE)
    {
        $intgtrabitproimposId = array (
            "cdimposto" => $itemProdutoImpostoNFE->getCdimpostoit(),
            "nrorg"       => \Geral\Util\Ambiente::getNrorg(),
        );

        return Imposto::getObject($intgtrabitproimposId);
    }


    /**
     * @static
     * @param $nrorg
     * @param $cdForneced
     * @param $cdSerie
     * @param $nrNotaFisc
     * @param $cdTipoOperacao
     *
     * @return \Fiscal\Entities\Notafiscal
     */
    public static function importa($nrorg, $cdForneced, $cdSerie, $nrNotaFisc, $cdTipoOperacao)
    {
        $importador = new static($nrorg, $cdForneced, $cdSerie, $nrNotaFisc);
        return $importador->importaDocumento($cdTipoOperacao);
    }

    /**
     * @return \Fiscal\Entities\Notafiscal|null
     */
    public function getNotaFiscal()
    {
        return $this->notaFiscal;
    }

}
