package com.javaseleniumtemplate.pages;

import com.javaseleniumtemplate.bases.PageBase;
import org.openqa.selenium.By;
import org.openqa.selenium.Keys;
import org.openqa.selenium.interactions.Actions;
import org.testng.Assert;

public class PesoProdutoUnidadePage extends PageBase {

    By botaoMenu = By.xpath("//div[@class='bars']");
    By campoPesquisar = By.xpath("//section[@class='zh-container-search-menu']//input[@type='text']");
    By opcaoTelaPesoProdutoUnidade = By.xpath("//span[text() = 'Peso do Produto na Unidade']");

    By unidadeDepositoJabaquara = By.xpath("//div[@data-zh-id=\"4526938862706767285688\"]//div[@class=\"list-items show-select\"]//span[text() = '0002 | DEPOSITO JABAQUARA']");
    By unidadeSelecionadaTopoDaTela = By.xpath("//aside[@data-zh-id=\"4526938863646256914685\"]//span[@class=\"stripe-value-NMFILIAL\"]");

    By codigoProdutoLinha1 = By.xpath("//div[@id=\"grid-4526938861321343098687\"]//div[@class=\"body-content\"]/div[1]//div[@data-zh-field-name=\"NVLCDARVPROD\"]/span");
    By nomeprodutoLinha1 = By.xpath("//div[@id=\"grid-4526938861321343098687\"]//div[@class=\"body-content\"]/div[1]//div[@data-zh-field-name=\"NMPRODUTO\"]/span");
    By campoPesoProdutoLinha1 = By.xpath("//div[@id=\"grid-4526938861321343098687\"]//div[@class=\"body-content\"]/div[1]//div[@data-zh-field-name=\"VRPESOUNIDFI\"]/span");

    By campoProcurar = By.xpath("//li[@class=\"float-action expandable fix-position-bottom opened\"]//input[@placeholder=\"Procurar\"]");;
    By botaoCampoUnidade = By.xpath("//div[@data-zh-id=\"4526938862706767285688\"]//div[@name=\"NMFILIAL\"]//span[@style=\"background-image: none; display: block;\"]");
    By botaoAplicarFiltro = By.xpath("//section[@class=\"popup-container\"]//div[@class=\"zh-footer-right\"]");
    By botaoDeOpcoesDaTela = By.xpath("//div[@data-ng-if=\"widget.isFloatingControlVisible()\"]//div[@data-zh-floating-control]");
    By botaoPesquisar = By.xpath("//div[@class=\"control-menu\"]//li[@data-zh-id=\"zh-floating-control-custom-action-1\"]");
    By botaoSalvarAlteracoes = By.xpath("//div[@class=\"zh-footer-right\"]");

    By mensagemCantoSuperiorDireito = By.xpath("//*[@class=\"zh-notification-messages\"]//span[@class=\"notification-message-text\"]");
    By mensagemObrigatorioUnidade = By.xpath("//div[@data-zh-id=\"4526938862706767285688\"]//div[@class=\"zh-validation\"]/span");

    public void acessarTela() {
        tempoParaAguardarTelaInicial();
        waitForElement(botaoMenu);
        click(botaoMenu);
        sendKeys(campoPesquisar, "Peso do Produto na Unidade");
        click(opcaoTelaPesoProdutoUnidade);
    }

    public void selecionarUnidadeDepositoJabaquara() {
        tempo();
        click(unidadeDepositoJabaquara);
        click(botaoAplicarFiltro);
        String unidade = getText(unidadeSelecionadaTopoDaTela);
        String unidadeEsperada = "0002 | DEPOSITO JABAQUARA";
        Assert.assertEquals(unidade, unidadeEsperada, "Verificar se a unidade selecionada é a que aparece no topo da tela após aplicar o filtro");
    }

    public void inserirPesoProduto() {
        preencherCampoProcurarTelaInicial("101110200601");
        tempo();
        String codigo = getText(codigoProdutoLinha1);
        String nome = getText(nomeprodutoLinha1);
        Assert.assertEquals(codigo, "1.01.1.1.02.006.01", "Verificar código do produto que foi encontrado pelo filtro");
        Assert.assertEquals(nome, "CARBONATO DE MAGNESIO 500G", "Verificar nome do produto que foi encontrado pelo filtro");
        alterarValorDoPeso("5");

        String mensagem = getText(mensagemCantoSuperiorDireito);
        String mensagemEsperada = "Salvo com sucesso.";
        Assert.assertEquals(mensagem, mensagemEsperada, "Verificar mensagem de salvo com sucesso, exibida após inserir um peso para o produto");

        //peso encontrado
        tempo();
        String encPeso = getText(campoPesoProdutoLinha1);
        Assert.assertEquals(encPeso, "5,000", "Verificar se o peso do produto foi salvo com sucesso");
    }

    public void editarPesoProduto() {
        preencherCampoProcurarTelaInicial("101110201502");
        tempo();
        String codigo = getText(codigoProdutoLinha1);
        String nome = getText(nomeprodutoLinha1);
        Assert.assertEquals(codigo, "1.01.1.1.02.015.02", "Verificar código do produto que foi encontrado pelo filtro");
        Assert.assertEquals(nome, "IODETO DE CALCIO 500G", "Verificar nome do produto que foi encontrado pelo filtro");
        alterarValorDoPeso("5,9");
        String mensagem = getText(mensagemCantoSuperiorDireito);
        String mensagemEsperada = "Salvo com sucesso.";
        Assert.assertEquals(mensagem, mensagemEsperada, "Verificar mensagem de salvo com sucesso após alterar o peso do produto");

        refresh();
        selecionarUnidadeDepositoJabaquara();
        preencherCampoProcurarTelaInicial("101110201502");
        tempo();
        String encPeso = getText(campoPesoProdutoLinha1);
        Assert.assertEquals(encPeso, "5,900", "Verificar se o valor do peso do produto foi salvo corretamente");

        preencherCampoProcurarTelaInicial("101110201502");
        tempo();
        alterarValorDoPeso("7");
        tempo();
        encPeso = getText(campoPesoProdutoLinha1);
        Assert.assertEquals(encPeso, "7,000", "Verificar se o valor do peso do produto foi salvo corretamente, retornando para o valor que estava no início do teste");
    }

    public void removerPesoProduto() {
        preencherCampoProcurarTelaInicial("101110200601");
        alterarValorDoPeso("");
        String mensagem = getText(mensagemCantoSuperiorDireito);
        String mensagemEsperada = "Salvo com sucesso.";
        Assert.assertEquals(mensagem, mensagemEsperada, "Verificar mensagem de salvo com sucesso, exibida após salvar um produto com o peso vazio");

        //peso encontrado
        String encPeso = getText(campoPesoProdutoLinha1);
        //peso esperado
        String expPeso = "";
        Assert.assertEquals(encPeso, expPeso, "Verificar se o peso do produto com o campo vazio foi salvo com sucesso");
    }

    public void campoObrigatorioUnidade() {
        click(unidadeDepositoJabaquara);
        click(botaoCampoUnidade);
        click(botaoAplicarFiltro);

        String unidade = getText(mensagemObrigatorioUnidade);
        String mensagemEsperada = "Campo obrigatório";
        Assert.assertEquals(unidade, mensagemEsperada, "Verificar mensagem de campo obrigatório no campo unidade do filtro inicial");
    }

    private void alterarValorDoPeso(String peso) {
        Actions actions = new Actions(driver);
        actions.click(driver.findElement(campoPesoProdutoLinha1)).build().perform();
        tempo();
        actions.sendKeys(Keys.BACK_SPACE).build().perform();
        tempo();
        actions.sendKeys(peso).build().perform();
        tempo();
        actions.sendKeys(Keys.ENTER).build().perform();
        tempo();
        click(botaoSalvarAlteracoes);
    }

    private void preencherCampoProcurarTelaInicial(String texto) {
        tempo();
        tempo();
        tempo();
        tempo();
        tempo();
        tempo();
        if (returnClassOpened(botaoDeOpcoesDaTela) && !returnClassOpened(botaoPesquisar)) {
            click(botaoPesquisar);
        } else if (!returnClassOpened(botaoDeOpcoesDaTela)){
            click(botaoDeOpcoesDaTela);
            click(botaoPesquisar);
        }
        clearAndSendKeys(campoProcurar, texto);
        tempo();
    }

    private boolean returnClassOpened(By locator) {
        return driver.findElement(locator).getAttribute("className").contains("opened");
    }

}
